
#include <Windows.h>
#include <cstdio>
#include <cstring>
#include <vector>
#include <ctime>
#include <cstdlib>
#include <algorithm>

#define ID_OBUTTON    1001
#define ID_XBUTTON    1002
#define ID_NEXTBUTTON 1003
#define ID_BACKBUTTON 1004
#define ID_FOURBUTTON 1010

#define QTYPE_OX   0
#define QTYPE_FOUR 1

using namespace std;

namespace myqma {

    struct QuestionData {
        int Type;
        wchar_t Text[1000];
        wchar_t Choices[4][100];
        int TextLength;
        int Answer;
    };

    vector<QuestionData> Questions;
    int QuestionNumber = 0;
    LPCWSTR ClassName = L"wc_myqma";
    LPCWSTR ApplicationName = L"QMAKvO";
    LPCWSTR QuestionFileName = L"question.txt";
    vector<int> IndexShuffler;

    void SetupQuestion( HWND window ) {
        random_shuffle( IndexShuffler.begin(), IndexShuffler.end() );
        ShowWindow( GetDlgItem( window, ID_OBUTTON ), SW_HIDE );
        ShowWindow( GetDlgItem( window, ID_XBUTTON ), SW_HIDE );
        for( int i=0; i<4; i++ )
            ShowWindow( GetDlgItem( window, ID_FOURBUTTON+i ), SW_HIDE );
        switch( Questions[QuestionNumber].Type ) {
        case QTYPE_OX:
            ShowWindow( GetDlgItem( window, ID_OBUTTON ), SW_SHOW );
            ShowWindow( GetDlgItem( window, ID_XBUTTON ), SW_SHOW );
            break;
        case QTYPE_FOUR:
            for( int i=0; i<4; i++ ) {
                ShowWindow( GetDlgItem( window, ID_FOURBUTTON+i ), SW_SHOW );
                SetWindowText( GetDlgItem( window, ID_FOURBUTTON+i ), Questions[QuestionNumber].Choices[IndexShuffler[i]] );
            }
            break;
        }
        InvalidateRect( window, NULL, TRUE );
    }

    // EBhEvV[W
    LRESULT CALLBACK WndProc( HWND window, UINT message, WPARAM wp, LPARAM lp ) {
        int button;
        switch( message ) {
        case WM_DESTROY:
            PostQuitMessage( 0 );
            break;
        case WM_COMMAND: // {^ꂽ̒ʒm
            button = LOWORD(wp);
            switch( Questions[QuestionNumber].Type ) {
            case QTYPE_OX:
                if( button==ID_OBUTTON&&Questions[QuestionNumber].Answer==0 ||
                    button==ID_XBUTTON&&Questions[QuestionNumber].Answer==1 ) {
                    MessageBox( window, L"I", ApplicationName, MB_OK );
                } else if( button==ID_OBUTTON&&Questions[QuestionNumber].Answer==1 ||
                           button==ID_XBUTTON&&Questions[QuestionNumber].Answer==0 ) {
                    MessageBox( window, L"s...", ApplicationName, MB_OK );
                }
                break;
            case QTYPE_FOUR:
                if( ID_FOURBUTTON<=button && button<=(ID_FOURBUTTON+3) ) {
                    int number = button - ID_FOURBUTTON;
                    if( IndexShuffler[number] == 0 ) {
                        MessageBox( window, L"I", ApplicationName, MB_OK );
                    } else {
                        MessageBox( window, L"s...", ApplicationName, MB_OK );
                    }
                }
                break;
            }
            switch( LOWORD(wp) ) {
            case ID_NEXTBUTTON:
                ++QuestionNumber;
                if( QuestionNumber >= (int)Questions.size() ) QuestionNumber = Questions.size() - 1;
                SetupQuestion( window );
                break;
            case ID_BACKBUTTON:
                --QuestionNumber;
                if( QuestionNumber < 0 ) QuestionNumber = 0;
                SetupQuestion( window );
                break;
            }
            break;
        case WM_PAINT: // 蕶`悷
            {
                PAINTSTRUCT ps;
                HDC hdc;
                hdc = BeginPaint( window, &ps );
                RECT r = { 15, 15, 400-15, 100 };
                DrawText( hdc, Questions[QuestionNumber].Text, Questions[QuestionNumber].TextLength, &r, DT_LEFT|DT_WORDBREAK );
                EndPaint( window, &ps );
            }
            break;
        default:
            return DefWindowProc( window, message, wp, lp );
        }
        return 0;
    }

    // t@C蕶ǂݍ
    bool LoadQuestionFile() {
        int i, j, k;
        FILE *fp;
        wchar_t temporary[1000];
        _wfopen_s( &fp, QuestionFileName, L"rt, ccs=UTF-16LE" );
        if( fp == NULL ) {
            MessageBox( NULL, L"t@Cǂݍ߂܂ł", ApplicationName, MB_OK );
            return false;
        }
        QuestionData item;
        Questions.clear();
        while( !feof( fp ) ) {
            fwscanf_s( fp, L"%d\n", &item.Type );
            if( NULL == fgetws( item.Text, sizeof(item.Text)/sizeof(item.Text[0]), fp ) )
                break;
            item.TextLength = wcsnlen( item.Text, sizeof(item.Text)/sizeof(item.Text[0]) );
            // Ō̉s菜
            item.Text[item.TextLength--] = L'\0';
            if( NULL == fgetws( temporary, sizeof(temporary)/sizeof(temporary[0]), fp ) )
                break;
            int temp_len = wcsnlen( temporary, sizeof(temporary)/sizeof(temporary[0]) );
            switch( item.Type ) {
            case QTYPE_OX:
                if( temporary[0] == L'0' ) {
                    item.Answer = 0;
                } else if( temporary[0] == L'1' ) {
                    item.Answer = 1;
                } else {
                    MessageBox( NULL, L"̒`sł", ApplicationName, MB_OK );
                    fclose( fp );
                    return false;
                }
                break;
            case QTYPE_FOUR:
                for( i=0, j=0, k=0; i<temp_len; ++i ) {
                    if( temporary[i] == L',' ) {
                        item.Choices[j][k] = L'\0';
                        ++j;
                        if( j==4 ) break;
                        k=0;
                        continue;
                    } else if( temporary[i] == L'\0' || temporary[i] == L'\n' ) {
                        item.Choices[j][k] = L'\0';
                        ++j;
                        break; 
                    }
                    item.Choices[j][k++] = temporary[i];
                }
                if( j!=4 ) {
                    MessageBox( NULL, L"lNCY̑I܂", ApplicationName, MB_OK );
                    fclose( fp );
                    return false;
                }
                item.Answer = 0;
                break;
            default:
                MessageBox( NULL, L"`ԍsł", ApplicationName, MB_OK );
                fclose( fp );
                return false;
            }
            Questions.push_back( item );
        }
        fclose( fp );
        if( Questions.size() == 0 ) {
            MessageBox( NULL, L"肪1܂", ApplicationName, MB_OK );
            return false;
        }
        return true;
    }

    // EBhEE\
    HWND PrepareWindow( HINSTANCE instance, int cmdshow ) {
        // EBhENX̍쐬
        WNDCLASSEX windowclass;
        ZeroMemory( &windowclass, sizeof(windowclass) );
        windowclass.cbSize = sizeof(windowclass);
        windowclass.hInstance = instance;
        windowclass.lpszClassName = ClassName;
        windowclass.lpfnWndProc = WndProc;
        windowclass.hbrBackground = (HBRUSH)GetStockObject( WHITE_BRUSH );
        windowclass.hCursor = LoadCursor( NULL, IDC_ARROW );
        if( 0 == RegisterClassEx( &windowclass ) ) {
            MessageBox( NULL, L"EBhENX̓o^Ɏs", ApplicationName, MB_OK );
            return NULL;
        }
        // EBhE̍쐬
        HWND mywindow;
        RECT windowrect = { 0, 0, 400, 300 };
        AdjustWindowRect( &windowrect, WS_OVERLAPPEDWINDOW, FALSE );
        mywindow = CreateWindow( ClassName,
            ApplicationName,
            WS_OVERLAPPEDWINDOW,
            CW_USEDEFAULT, CW_USEDEFAULT,
            windowrect.right-windowrect.left, windowrect.bottom-windowrect.top,
            NULL, NULL, instance, NULL );
        CreateWindow( L"BUTTON", L"", WS_CHILD|WS_VISIBLE,
             80, 120, 100, 100, mywindow, (HMENU)ID_OBUTTON, instance, NULL );
        CreateWindow( L"BUTTON", L"~", WS_CHILD|WS_VISIBLE,
            220, 120, 100, 100, mywindow, (HMENU)ID_XBUTTON, instance, NULL );
        CreateWindow( L"BUTTON", L" >>", WS_CHILD|WS_VISIBLE,
            400-90, 300-40, 80, 30, mywindow, (HMENU)ID_NEXTBUTTON, instance, NULL );
        CreateWindow( L"BUTTON", L"<< O", WS_CHILD|WS_VISIBLE,
            400-90-90, 300-40, 80, 30, mywindow, (HMENU)ID_BACKBUTTON, instance, NULL );
        for( int i=0; i<4; i++ ) {
            CreateWindow( L"BUTTON", L"I", WS_CHILD|WS_VISIBLE,
                50, 100+40*i, 300, 30, mywindow, (HMENU)(ID_FOURBUTTON+i), instance, NULL );
        }
        ShowWindow( mywindow, cmdshow|SW_SHOW );
        return mywindow;
    }

    extern "C" int WINAPI wWinMain( HINSTANCE instance, HINSTANCE prev_instance, LPWSTR commandline, int cmdshow ) {
        HWND mainwindow;
        if( ! LoadQuestionFile() )
            return 1;
        mainwindow = PrepareWindow( instance, cmdshow );
        if( mainwindow == NULL )
            return 1;
        srand( GetTickCount() );
        for( int i=0; i<4; i++ ) IndexShuffler.push_back( i );
        SetupQuestion( mainwindow );
        // bZ[W[v
        MSG message;
        while( GetMessage( &message, NULL, 0, 0 ) ) {
            DispatchMessage( &message );
        }
        DestroyWindow( mainwindow );
        return message.wParam;
    }
}

